@@ -4,52 +4,13 @@ use std::{fmt, ops};
 
 use tract_data::prelude::*;
 
+use super::*;
 use crate::frame::mmm::LinearSpec::*;
 use crate::frame::mmm::PanelStore::*;
 use crate::frame::mmm::*;
 
 use num_traits::sign::Signed;
 
-pub trait PseudoRightShift {
-    fn q_away(self, mult: Self, shift: usize) -> Self;
-    fn q_even(self, mult: Self, shift: usize) -> Self;
-    fn q_to_plus_inf(self, mult: Self, shift: usize) -> Self;
-}
-
-impl PseudoRightShift for i32 {
-    fn q_even(self, mult: Self, shift: usize) -> Self {
-        let v = ((self as i64 * mult as i64) >> (30 + shift)) as i32;
-        let truncated = v.abs();
-        let nudge = ((truncated & 0x3) == 0x3) as usize as i32;
-        let pos = (truncated + nudge) >> 1;
-        if v.is_negative() {
-            -pos
-        } else {
-            pos
-        }
-    }
-    fn q_to_plus_inf(self, mult: Self, shift: usize) -> Self {
-        let v = ((self as i64 * mult as i64) >> (30 + shift)) as i32;
-        (v + 1) >> 1
-    }
-    fn q_away(self, mult: Self, shift: usize) -> Self {
-        let v = ((self.abs() as i64 * mult as i64) >> (30 + shift)) as i32;
-        ((v + 1) >> 1) * self.signum()
-    }
-}
-
-impl PseudoRightShift for f32 {
-    fn q_even(self, mult: Self, shift: usize) -> Self {
-        self * mult * 2f32.powi(-(shift as i32))
-    }
-    fn q_to_plus_inf(self, mult: Self, shift: usize) -> Self {
-        self * mult * 2f32.powi(-(shift as i32))
-    }
-    fn q_away(self, mult: Self, shift: usize) -> Self {
-        self * mult * 2f32.powi(-(shift as i32))
-    }
-}
-
 #[derive(Copy, Clone, Debug)]
 pub struct GenericMmm4x4<TA, TB, TC, TI>(PhantomData<(TA, TB, TC, TI)>)
 where
@@ -61,7 +22,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -79,7 +40,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -99,7 +60,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -119,7 +80,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + Signed
@@ -293,29 +254,15 @@ where
                             }
                         }
                     }
-                    FusedKerSpec::QTowardsEven(mult, shift) => {
-                        for i in 0..4 {
-                            for j in 0..4 {
-                                ab[i][j] = ab[i][j].q_even(mult, shift);
-                            }
-                        }
-                    }
-                    FusedKerSpec::QTowardsPlusInf(mult, shift) => {
-                        for i in 0..4 {
-                            for j in 0..4 {
-                                ab[i][j] = ab[i][j].q_to_plus_inf(mult, shift);
-                            }
-                        }
-                    }
-                    FusedKerSpec::QAway(mult, shift) => {
-                        for i in 0..4 {
-                            for j in 0..4 {
-                                ab[i][j] = ab[i][j].q_away(mult, shift);
-                            }
-                        }
-                    }
                     FusedKerSpec::AddUnicast(tile) => add_unicast::<TC, TI, _>(&tile, &mut ab),
-                }
+                    FusedKerSpec::QScale(shift, rp, mult) => {
+                        for i in 0..4 {
+                            for j in 0..4 {
+                                ab[i][j] = ab[i][j].q_scale(mult, shift, rp);
+                            }
+                        }
+                    }
+                };
                 pnl = pnl.add(1);
             }
             let Tile { ptr: c, row_byte_stride, col_byte_stride, .. } = spec.c;
@@ -356,7 +303,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -374,7 +321,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -394,7 +341,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + fmt::Debug
@@ -414,7 +361,7 @@ where
         + ops::AddAssign
         + ops::Mul<Output = TI>
         + ops::MulAssign
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + PartialOrd
         + Zero
         + Signed
@@ -539,21 +486,6 @@ where
                             ab[i] *= a;
                         }
                     }
-                    FusedKerSpec::QTowardsEven(mult, shift) => {
-                        for i in 0..4 {
-                            ab[i] = ab[i].q_even(mult, shift);
-                        }
-                    }
-                    FusedKerSpec::QTowardsPlusInf(mult, shift) => {
-                        for i in 0..4 {
-                            ab[i] = ab[i].q_to_plus_inf(mult, shift);
-                        }
-                    }
-                    FusedKerSpec::QAway(mult, shift) => {
-                        for i in 0..4 {
-                            ab[i] = ab[i].q_away(mult, shift);
-                        }
-                    }
                     FusedKerSpec::AddUnicast(tile) => add_unicast::<TC, TI, _>(
                         &tile,
                         &mut [
@@ -563,6 +495,11 @@ where
                             std::slice::from_raw_parts_mut(ab.as_ptr().offset(3) as _, 1),
                         ],
                     ),
+                    FusedKerSpec::QScale(shift, rp, mult) => {
+                        for i in 0..4 {
+                            ab[i] = ab[i].q_scale(mult, shift, rp);
+                        }
+                    }
                 }
                 pnl = pnl.add(1);
             }
@@ -650,7 +587,7 @@ where
         + ops::Mul<Output = TI>
         + ops::MulAssign
         + PartialOrd
-        + PseudoRightShift
+        + ScaleShiftAndRound
         + Zero
         + Signed
         + fmt::Debug
@@ -793,28 +730,14 @@ where
                             }
                         }
                     }
-                    FusedKerSpec::QTowardsEven(mult, shift) => {
-                        for i in 0..3 {
-                            for j in 0..2 {
-                                ab[i][j] = ab[i][j].q_even(mult, shift);
-                            }
-                        }
-                    }
-                    FusedKerSpec::QTowardsPlusInf(mult, shift) => {
-                        for i in 0..3 {
-                            for j in 0..2 {
-                                ab[i][j] = ab[i][j].q_to_plus_inf(mult, shift);
-                            }
-                        }
-                    }
-                    FusedKerSpec::QAway(mult, shift) => {
-                        for i in 0..3 {
-                            for j in 0..2 {
-                                ab[i][j] = ab[i][j].q_away(mult, shift);
-                            }
-                        }
-                    }
                     FusedKerSpec::AddUnicast(tile) => add_unicast::<TC, TI, _>(&tile, &mut ab),
+                    FusedKerSpec::QScale(shift, rp, mult) => {
+                        for i in 0..3 {
+                            for j in 0..2 {
+                                ab[i][j] = ab[i][j].q_scale(mult, shift, rp);
+                            }
+                        }
+                    }
                 }
                 pnl = pnl.add(1);
             }
